package com.adzel.addyauthreborn;

import com.adzel.addyauthreborn.auth.AuthManager;
import com.adzel.addyauthreborn.commands.LoginCommand;
import com.adzel.addyauthreborn.commands.LockCommand;
import com.adzel.addyauthreborn.commands.RegisterCommand;
import com.adzel.addyauthreborn.commands.AddyAuthCommand;
import com.adzel.addyauthreborn.listeners.LockListeners;
import com.adzel.addyauthreborn.listeners.AdminJoinListener;
import com.adzel.addyauthreborn.listeners.ConnectionListener;
import com.adzel.addyauthreborn.lock.AFKMonitor;
import com.adzel.addyauthreborn.lock.OfflineLockManager;
import com.adzel.addyauthreborn.updates.UpdateChecker;
import com.adzel.addyauthreborn.utils.DatabaseUtil;
import com.zaxxer.hikari.HikariDataSource;
import net.kyori.adventure.platform.bukkit.BukkitAudiences;
import org.bukkit.plugin.java.JavaPlugin;

public class AddyAuthReborn extends JavaPlugin {

    private static AddyAuthReborn instance;
    private AuthManager authManager;
    private AFKMonitor afkMonitor;
    private OfflineLockManager offlineLockManager;
    private BukkitAudiences adventure;
    private boolean updateAvailable = false;

    @Override
    public void onEnable() {
        instance = this;

        saveDefaultConfig();

        adventure = BukkitAudiences.create(this);

        if (!DatabaseUtil.initDatabase()) {
            getLogger().severe("Failed to initialize database. Disabling plugin.");
            getServer().getPluginManager().disablePlugin(this);
            return;
        }

        authManager = new AuthManager();
        afkMonitor = new AFKMonitor(this);
        offlineLockManager = new OfflineLockManager(this);

        // Commands
        getCommand("register").setExecutor(new RegisterCommand(this));
        getCommand("login").setExecutor(new LoginCommand(this));
        getCommand("lock").setExecutor(new LockCommand(this));
        getCommand("addyauth").setExecutor(new AddyAuthCommand(this));

        // Listeners
        getServer().getPluginManager().registerEvents(new LockListeners(this), this);
        getServer().getPluginManager().registerEvents(new AdminJoinListener(this), this);
        getServer().getPluginManager().registerEvents(new ConnectionListener(this), this); // PlayerJoin/Leave for tracking AFK state

        // Update check
        new UpdateChecker(this, "https://git.adzeldevelops.site/api/v1/repos/AdzelFirestar/AddyAuthReborn/releases/latest")
                .getLatestVersion(latest -> {
                    String current = getDescription().getVersion();
                    if (!current.equalsIgnoreCase(latest)) {
                        updateAvailable = true;
                        getLogger().warning("A new version of AddyAuth Reborn is available: " + latest);
                    }
                });

        getLogger().info("AddyAuth Reborn enabled successfully.");
    }

    @Override
    public void onDisable() {
        if (adventure != null) {
            adventure.close();
        }
        if (afkMonitor != null) {
            afkMonitor.shutdown(); // 🧼 clean up task
        }
        DatabaseUtil.shutdown();
        getLogger().info("AddyAuth Reborn has been disabled.");
    }

    public static AddyAuthReborn getInstance() {
        return instance;
    }

    public AuthManager getAuthManager() {
        return authManager;
    }

    public AFKMonitor getAFKMonitor() {
        return afkMonitor;
    }

    public OfflineLockManager getOfflineLockManager() {
        return offlineLockManager;
    }

    public HikariDataSource getDataSource() {
        return DatabaseUtil.getDataSource();
    }

    public boolean isUpdateAvailable() {
        return updateAvailable;
    }

    public int getMinPasswordLength() {
        return getConfig().getInt("General Settings.Minimum Password Length", 5);
    }

    public int getMaxPasswordLength() {
        return getConfig().getInt("General Settings.Maximum Password Length", 15);
    }

    public BukkitAudiences adventure() {
        return adventure;
    }
}
